==> happy-css-modules-0.6.0.tgz: package/src/emitter/dts.ts <==
import { EOL } from 'os';
import { basename } from 'path';
import camelcase from 'camelcase';
import { SourceNode, type CodeWithSourceMap } from '../library/source-map/index.js';
import { type Token } from '../locator/index.js';
import { type LocalsConvention } from '../runner.js';
import { getRelativePath, type DtsFormatOptions } from './index.js';

/**
 * Get .d.ts file path.
 * @param filePath The path to the source file (i.e. `/dir/foo.css`). It is absolute.
 * @returns The path to the .d.ts file. It is absolute.
 */
export function getDtsFilePath(filePath: string): string {
  return filePath + '.d.ts';
}

function dashesCamelCase(str: string): string {
  return str.replace(/-+(\w)/g, function (match, firstLetter) {
    return firstLetter.toUpperCase();
  });
}

function formatTokens(tokens: Token[], localsConvention: LocalsConvention): Token[] {
  const result: Token[] = [];
  for (const token of tokens) {
    if (localsConvention === 'camelCaseOnly') {
      result.push({ ...token, name: camelcase(token.name) });
    } else if (localsConvention === 'camelCase') {
      result.push(token);
      result.push({ ...token, name: camelcase(token.name) });
    } else if (localsConvention === 'dashesOnly') {
      result.push({ ...token, name: dashesCamelCase(token.name) });
    } else if (localsConvention === 'dashes') {
      result.push(token);
      result.push({ ...token, name: dashesCamelCase(token.name) });
    } else {
      result.push(token); // asIs
    }
  }
  return result;
}

function generateTokenDeclarations(
  filePath: string,
  sourceMapFilePath: string,
  tokens: Token[],
  dtsFormatOptions: DtsFormatOptions | undefined,
  isExternalFile: (filePath: string) => boolean,
): typeof SourceNode[] {
  const formattedTokens = formatTokens(tokens, dtsFormatOptions?.localsConvention);
  const result: typeof SourceNode[] = [];

  for (const token of formattedTokens) {
    // Only one original position can be associated with one generated position.
    // This is due to the sourcemap specification. Therefore, we output multiple type definitions
    // with the same name and assign a separate original position to each.

    for (let originalLocation of token.originalLocations) {
      if (originalLocation.filePath === undefined) {
        // If the original location is not specified, fallback to the source file.
        originalLocation = {
          filePath: filePath,
          start: { line: 1, column: 1 },
          end: { line: 1, column: 1 },
        };
      }

      result.push(
        originalLocation.filePath === filePath || isExternalFile(originalLocation.filePath)
          ? new SourceNode(null, null, null, [
              '& Readonly<{ ',
              new SourceNode(
                originalLocation.start.line ?? null,
                // The SourceNode's column is 0-based, but the originalLocation's column is 1-based.
                originalLocation.start.column - 1 ?? null,
                getRelativePath(sourceMapFilePath, originalLocation.filePath),
                `"${token.name}"`,
                token.name,
              ),
              ': string }>',
            ])
          : // Imported tokens in non-external files are typed by dynamic import.
            // See https://github.com/mizdra/happy-css-modules/issues/106.
            new SourceNode(null, null, null, [
              '& Readonly<Pick<(typeof import(',
              `"${getRelativePath(filePath, originalLocation.filePath)}"`,
              '))["default"], ',
              `"${token.name}"`,
              '>>',
            ]),
      );
    }
  }
  return result;
}

export function generateDtsContentWithSourceMap(
  filePath: string,
  dtsFilePath: string,
  sourceMapFilePath: string,
  tokens: Token[],
  dtsFormatOptions: DtsFormatOptions | undefined,
  isExternalFile: (filePath: string) => boolean,
): { dtsContent: CodeWithSourceMap['code']; sourceMap: CodeWithSourceMap['map'] } {
  const tokenDeclarations = generateTokenDeclarations(
    filePath,
    sourceMapFilePath,
    tokens,
    dtsFormatOptions,
    isExternalFile,
  );

  let sourceNode: typeof SourceNode;
  if (!tokenDeclarations || !tokenDeclarations.length) {
    sourceNode = new SourceNode(null, null, null, '');
  } else {
    sourceNode = new SourceNode(1, 0, getRelativePath(sourceMapFilePath, filePath), [
      'declare const styles:' + EOL,
      ...tokenDeclarations.map((tokenDeclaration) => ['  ', tokenDeclaration, EOL]),
      ';' + EOL,
      'export default styles;' + EOL,
    ]);
  }
  const codeWithSourceMap = sourceNode.toStringWithSourceMap({
    // Since sourcemap and type definitions are in the same directory, they can be referenced by relative paths.
    file: basename(dtsFilePath),
    sourceRoot: '',
  });
  return {
    dtsContent: codeWithSourceMap.code,
    sourceMap: codeWithSourceMap.map,
  };
}

==> happy-css-modules-2.1.2.tgz: package/src/emitter/dts.ts <==
import { EOL } from 'os';
import { basename, parse, join } from 'path';
import camelcase from 'camelcase';
import { SourceNode, type CodeWithSourceMap } from '../library/source-map/index.js';
import { type Token } from '../locator/index.js';
import { type LocalsConvention } from '../runner.js';
import { getRelativePath, type DtsFormatOptions } from './index.js';

/**
 * Get .d.ts file path.
 * @param filePath The path to the source file (i.e. `/dir/foo.css`). It is absolute.
 * @param arbitraryExtensions Generate `.d.css.ts` instead of `.css.d.ts`.
 * @returns The path to the .d.ts file. It is absolute.
 */
export function getDtsFilePath(filePath: string, arbitraryExtensions: boolean): string {
  if (arbitraryExtensions) {
    const { dir, name, ext } = parse(filePath);
    return join(dir, name + '.d' + ext + '.ts');
  } else {
    return filePath + '.d.ts';
  }
}

function dashesCamelCase(str: string): string {
  return str.replace(/-+(\w)/g, function (match, firstLetter) {
    return firstLetter.toUpperCase();
  });
}

function formatTokens(tokens: Token[], localsConvention: LocalsConvention): Token[] {
  const result: Token[] = [];
  for (const token of tokens) {
    if (localsConvention === 'camelCaseOnly') {
      result.push({ ...token, name: camelcase(token.name) });
    } else if (localsConvention === 'camelCase') {
      result.push(token);
      result.push({ ...token, name: camelcase(token.name) });
    } else if (localsConvention === 'dashesOnly') {
      result.push({ ...token, name: dashesCamelCase(token.name) });
    } else if (localsConvention === 'dashes') {
      result.push(token);
      result.push({ ...token, name: dashesCamelCase(token.name) });
    } else {
      result.push(token); // asIs
    }
  }
  return result;
}

function generateTokenDeclarations(
  filePath: string,
  sourceMapFilePath: string,
  tokens: Token[],
  dtsFormatOptions: DtsFormatOptions | undefined,
  isExternalFile: (filePath: string) => boolean,
): typeof SourceNode[] {
  const formattedTokens = formatTokens(tokens, dtsFormatOptions?.localsConvention);
  const result: typeof SourceNode[] = [];

  for (const token of formattedTokens) {
    // Only one original position can be associated with one generated position.
    // This is due to the sourcemap specification. Therefore, we output multiple type definitions
    // with the same name and assign a separate original position to each.

    for (let originalLocation of token.originalLocations) {
      if (originalLocation.filePath === undefined) {
        // If the original location is not specified, fallback to the source file.
        originalLocation = {
          filePath: filePath,
          start: { line: 1, column: 1 },
          end: { line: 1, column: 1 },
        };
      }

      result.push(
        originalLocation.filePath === filePath || isExternalFile(originalLocation.filePath)
          ? new SourceNode(null, null, null, [
              '& Readonly<{ ',
              new SourceNode(
                originalLocation.start.line ?? null,
                // The SourceNode's column is 0-based, but the originalLocation's column is 1-based.
                originalLocation.start.column - 1 ?? null,
                getRelativePath(sourceMapFilePath, originalLocation.filePath),
                `"${token.name}"`,
                token.name,
              ),
              ': string }>',
            ])
          : // Imported tokens in non-external files are typed by dynamic import.
            // See https://github.com/mizdra/happy-css-modules/issues/106.
            new SourceNode(null, null, null, [
              '& Readonly<Pick<(typeof import(',
              `"${getRelativePath(filePath, originalLocation.filePath)}"`,
              '))["default"], ',
              `"${token.name}"`,
              '>>',
            ]),
      );
    }
  }
  return result;
}

export function generateDtsContentWithSourceMap(
  filePath: string,
  dtsFilePath: string,
  sourceMapFilePath: string,
  tokens: Token[],
  dtsFormatOptions: DtsFormatOptions | undefined,
  isExternalFile: (filePath: string) => boolean,
): { dtsContent: CodeWithSourceMap['code']; sourceMap: CodeWithSourceMap['map'] } {
  const tokenDeclarations = generateTokenDeclarations(
    filePath,
    sourceMapFilePath,
    tokens,
    dtsFormatOptions,
    isExternalFile,
  );

  let sourceNode: typeof SourceNode;
  if (!tokenDeclarations || !tokenDeclarations.length) {
    sourceNode = new SourceNode(null, null, null, '');
  } else {
    sourceNode = new SourceNode(1, 0, getRelativePath(sourceMapFilePath, filePath), [
      'declare const styles:' + EOL,
      ...tokenDeclarations.map((tokenDeclaration) => ['  ', tokenDeclaration, EOL]),
      ';' + EOL,
      'export default styles;' + EOL,
    ]);
  }
  const codeWithSourceMap = sourceNode.toStringWithSourceMap({
    // Since sourcemap and type definitions are in the same directory, they can be referenced by relative paths.
    file: basename(dtsFilePath),
    sourceRoot: '',
  });
  return {
    dtsContent: codeWithSourceMap.code,
    sourceMap: codeWithSourceMap.map,
  };
}
